#include <GL/glut.h>
#include <stdlib.h>

#include "GameObj.h"
#include "GameObjTimed.h"
#include "Bullet.h"
#include "Ship.h"
#include "Ship3D.h"
#include "Ast.h"
#include "Ast3DFract.h"
#include "Exp.h"
#include "Fragment.h"
#include "GameSession.h"
#include "GameIntro.h"
#include "Stars.h"
#include "extern.h"

#ifdef _WIN32
#define drand48() (((float) rand())/((float) RAND_MAX))
#define srand48(x) (srand((x)))
#endif

void GameSession::Key(unsigned char key)
{
	switch(key){
		case ' ' : Shoot(); break;
		case 'p' : Pause(); break;
		}
}

void  GameSession::SKey(int key)
{
  int km = glutGetModifiers();
	switch (key) {
  case GLUT_KEY_LEFT: 
	  if(km & GLUT_ACTIVE_ALT) S->StartStrafeLeft();
			else S->StartLeft();
	  break;
	case GLUT_KEY_RIGHT: 
		if(km & GLUT_ACTIVE_ALT) S->StartStrafeRight();
			else  S->StartRight();
	  break;
  case GLUT_KEY_UP: 
	  S->ThrustOn();
	  break;
	}
}

void  GameSession::SKeyUp(int key)
{
 switch (key) {
  case GLUT_KEY_LEFT: 
	  S->StopStrafeLeft();
		S->StopLeft();
	  break;
  case GLUT_KEY_RIGHT: 
	  S->StopStrafeRight();
		S->StopRight();
	  break;
  case GLUT_KEY_UP: 
	  S->ThrustOff();
	  break;
	}
}
void  GameSession::Reshape(int w,int h)
{
  ScreenW=w;
  ScreenH=h;
	float wr=float(w)/h;
  float hr=float(h)/w;

	if(w>h) hr=1;
	else wr=1;
  int VL = -SpaceSize*wr;
  int VR =  SpaceSize*wr;
  int VB = -SpaceSize*hr;
  int VT =  SpaceSize*hr;
  glViewport(0,0,w,h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(VL,VR,VB,VT,-100,100);
  glMatrixMode(GL_MODELVIEW);
}
/** The main drawing function.
It draw the current ship, all the active bullets,
asteroids and  generic objects
*/
int GameSession::Draw()
{
	int nt=0;
	glPushMatrix();
	glScalef(SpaceSize*2,SpaceSize*2,0);
	nt+=SB.Draw();
	glPopMatrix();
	if(S->active)
		nt+=S->Draw();

	list<Bullet *>::iterator lib;
	for(lib=ActiveBullet.begin();lib!=ActiveBullet.end();++lib)
		nt+=(*lib)->Draw();
	list<Ast *>::iterator lia;
	for(lia=ActiveAst.begin();lia!=ActiveAst.end();++lia)
		nt+=(*lia)->Draw();
	list<GameObj *>::iterator li;
	for(li=ActiveObj.begin();li!=ActiveObj.end();++li)
		nt+=(*li)->Draw();	
	return nt;
}

void GameSession::Update(int dt)
{
	SB.Update(dt);
	if(PauseFlag) return;
	if(S->active){
		S->Update(dt);
		Clip(S->p);
	} else {
		deadpause-=dt;
		if(deadpause<0){
			Game->End();
			Game=&GI;
			GI.Init();
			GI.Reshape(ScreenW,ScreenH);
			}
	}

	list<Bullet *>::iterator lib;
	for(lib=ActiveBullet.begin();lib!=ActiveBullet.end();++lib)
		{
		 (*lib)->Update(dt);
			Clip((*lib)->p);
			BulletAstCollision(*lib);
			if((*lib)->active==false)
			{
				GameObj *todel=*lib;
				lib=ActiveBullet.erase(lib);
				S->TerminateBullet();
				delete todel;
			}
			if(lib==ActiveBullet.end()) break;
		}
	
	list<Ast *>::iterator lia;
	for(lia=ActiveAst.begin();lia!=ActiveAst.end();++lia)
		{
		 (*lia)->Update(dt);
			Clip((*lia)->p);
			AstShipCollision(*lia);
			if((*lia)->active==false)
		 {
			 GameObj *todel=*lia;
			 lia=ActiveAst.erase(lia);
			 delete todel;
		 }
		 if(lia==ActiveAst.end()) break;
		}

	list<GameObj *>::iterator li;
	for(li=ActiveObj.begin();li!=ActiveObj.end();++li)
		{
		 (*li)->Update(dt);
			Clip((*li)->p);
		 if((*li)->active==false)
		 {
			 GameObj *todel=*li;
			 li=ActiveObj.erase(li);
			 delete todel;
		 }
		 if(li==ActiveObj.end()) break;
		}

}

void GameSession::Shoot()
{
 S->Shoot(ActiveBullet);
}

void GameSession::End()
{
	Exp::LightOffAll();
}
void GameSession::Init()
{ 
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	float lp[4]={1,1,3,0};
	glLightfv(GL_LIGHT0,GL_POSITION,lp);
  glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
	float lc[4]={1,.7,.5,1};
	for(int i=0;i<3;i++)
		{
			glLightfv(GL_LIGHT1+i,GL_DIFFUSE,lc);
			glLightf(GL_LIGHT1+i,GL_QUADRATIC_ATTENUATION,.001f);
		}	
	glEnable(GL_CULL_FACE);
	// Remove Comments if some face disappears...
	//glDisable(GL_CULL_FACE);
	//glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,1);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);

	ActiveAst.clear();
	ActiveBullet.clear();
	ActiveObj.clear();
	
	Exp::LightOffAll();
	S=new Ship3D("data/newship.rawc");
	Ast *a;
	for(i=0;i<AstNum;i++)
		{
			a=new Ast3DFract("data/tetra.raw",Ast::LargeSize);
			a->p.x()=drand48()*SpaceSize*2 - SpaceSize;	
			a->p.y()=drand48()*SpaceSize*2 - SpaceSize;
			a->p.z()=0;
			a->v.x()=drand48()*40 - 20;	
			a->v.y()=drand48()*40 - 20;
			a->v.z()=0;
			a->av=20+drand48()*70;
			a->axis=Point3f(drand48(),drand48(),drand48());
			a->GenerateSub();
			ActiveAst.push_back(a);
		}
}
void GameSession::AstShipCollision(Ast *a)
{
	if(S->active && a->Collide(S)) // Asteroid - Ship Collision
			{
				S->active=false;
				deadpause=2000;
				Exp *e =new Exp();
				e->p=(a->p+S->p)/2;
				ActiveObj.push_back(e);	
				Fragment *f;
				for(int i=0; i<50;i++)
				{
					if(i%2) f=new Fragment(Point3f(1,0,0));
					else f=new Fragment(Point3f(0,1,0));
					f->p=S->p;
					f->v=Point3f(drand48()-.5f,drand48()-.5f,0)*30;
					f->axis=Normalize(Point3f(drand48(),drand48(),drand48()));
					f->av=40+drand48()*100;
					f->ttl=2000;
					ActiveObj.push_front(f);
				}

			}
}

void GameSession::BulletAstCollision(Bullet *b)
{
	if(!b->active) return;
	list<Ast *>::iterator lia;
	for(lia=ActiveAst.begin();lia!=ActiveAst.end();++lia)
		if((*lia)->active && (*lia)->Collide(b))
			{
				b->active=false;
				(*lia)->Explode(ActiveAst,ActiveObj);
				return;
			}
}